from __future__ import print_function
import numpy as np
from molkrr.krr import read22k, timer, sorted_coulomb_matrix
from molkrr.calc import readjson, runkrr

nlearn = 3000
ntest = 500
seed = 0
R0 = 3000.0
gamma = 0.02
G0 = 10.0

gen = np.random.RandomState(seed)

with timer('xyz'):
    allsystems = read22k()  # largest system is 26 atoms


files = dict(ex1_pbe0='orca-pbe0.absspec.e_1_eV.json.gz',
             ex1_lda='orca-lda.absspec.e_1_eV.json.gz',
             gap_pbe0='orca-pbe0.gs.homolumogap.json.gz',
             gap_lda='orca-lda.gs.homolumogap.json.gz')

all_edata = {}
with timer('data'):
    for name in files:
        all_edata[name] = np.array(readjson(files[name]))

nallsys = len(allsystems)
indices = gen.choice(range(nallsys), nlearn + ntest, replace=False)

edata = {}
for name in all_edata:
    edata[name] = all_edata[name][indices]

systems = [allsystems[i] for i in indices]

def get_padded_scms(systems):
    maxlen = max(len(sys) for sys in allsystems)
    scms = np.zeros((len(systems), maxlen, maxlen))
    for i, system in enumerate(systems):
        n = len(system)
        scms[i, :n, :n] = sorted_coulomb_matrix(system).reshape(n, n)
    return scms.reshape(len(systems), -1)
with timer('scm'):
    scms = get_padded_scms(systems)

ilearn = indices[:nlearn]
itest = indices[nlearn:]
assert len(ilearn) == nlearn
assert len(itest) == ntest

def desc_cat(basedescs, sigmas):
    assert len(sigmas) == len(basedescs)
    nsys = len(basedescs[0])

    descs = []
    for i in range(nsys):
        d = []
        for sigma, basedesc in zip(sigmas, basedescs):
            if len(basedesc.shape) == 1:
                basedesc = basedesc[:, None]
            desc = basedesc[i] / sigma
            d.append(desc)
        descs.append(np.concatenate(d))
    return np.array(descs)

#basevalues = edata['gap_pbe0']
basevalues = edata['ex1_lda']
#basevalues = np.zeros_like(edata['ex1_lda'])
refs = edata['ex1_pbe0'] - basevalues

#refs = edata['ex1_pbe0']
#basevalues = np.zeros(len(itest))

descs = desc_cat([scms, edata['gap_pbe0']], [R0, G0])
#descs = desc_cat([scms], [R0])

#print(descs.shape)

learndesc = descs[:nlearn]
testdesc = descs[nlearn:]
learnref = refs[:nlearn]
testref = refs[nlearn:]
testbasevalues = basevalues[nlearn:]
assert len(learndesc) == nlearn
assert len(learnref) == nlearn
assert len(testdesc) == ntest
assert len(testref) == ntest

import numpy as np
np.set_printoptions(precision=2, suppress=1)

results = runkrr(learndescs=learndesc,
                 learnref=learnref,
                 testdescs=testdesc,
                 testref=testref,
                 gamma=gamma)
#print('mae', results['mae'], 'rms', results['rms'])

pred = results['pred']

import pylab as pl

#pl.plot(results['ref'], results['pred'], 'bo')

newpred = testbasevalues + pred
newbase = testbasevalues + testref
pl.plot(newbase, newpred, 'bo')#, 'rv')

errs = np.abs(newpred - newbase)

print('mae', errs.sum() / len(errs),
      'rms', np.sqrt((errs**2).sum() / len(errs)))

pl.show()
